[PATCH 02/11] should_swap

Matthew Auld matthew.auld at intel.com
Fri Sep 3 14:41:59 UTC 2021


---
 .../gpu/drm/i915/gem/selftests/huge_pages.c   | 24 +++++++++++++++----
 1 file changed, 19 insertions(+), 5 deletions(-)

diff --git a/drivers/gpu/drm/i915/gem/selftests/huge_pages.c b/drivers/gpu/drm/i915/gem/selftests/huge_pages.c
index acc435f14ac9..9a0ded31b4a7 100644
--- a/drivers/gpu/drm/i915/gem/selftests/huge_pages.c
+++ b/drivers/gpu/drm/i915/gem/selftests/huge_pages.c
@@ -1519,6 +1519,7 @@ static int igt_shrink_thp(void *arg)
 	struct i915_vma *vma;
 	unsigned int flags = PIN_USER;
 	unsigned int n;
+	bool should_swap;
 	int err = 0;
 
 	/*
@@ -1567,26 +1568,39 @@ static int igt_shrink_thp(void *arg)
 			break;
 	}
 	i915_gem_context_unlock_engines(ctx);
+	/*
+	 * Nuke everything *before* we unpin the pages so we can be sure when
+	 * later checking get_nr_swap_pages() that some random leftover object
+	 * steal the remaining swap space.
+	 */
+	i915_gem_shrink(NULL, i915, -1UL, NULL,
+			I915_SHRINK_BOUND |
+			I915_SHRINK_UNBOUND |
+			I915_SHRINK_ACTIVE);
 	i915_vma_unpin(vma);
 	if (err)
 		goto out_put;
 
+
 	/*
 	 * Now that the pages are *unpinned* shrinking should invoke
 	 * shmem to truncate our pages.
 	 */
+	should_swap = get_nr_swap_pages() > 0;
 	i915_gem_shrink(NULL, i915, -1UL, NULL,
 			I915_SHRINK_BOUND |
 			I915_SHRINK_UNBOUND |
 			I915_SHRINK_ACTIVE);
-	if (i915_gem_object_has_pages(obj)) {
-		pr_err("shrinking didn't truncate the pages\n");
+	if (should_swap == i915_gem_object_has_pages(obj)) {
+		pr_err("unexpected pages mismatch, should_swap=%s\n",
+		       yesno(should_swap));
 		err = -EINVAL;
 		goto out_put;
 	}
 
-	if (obj->mm.page_sizes.sg || obj->mm.page_sizes.phys) {
-		pr_err("residual page-size bits left\n");
+	if (should_swap == (obj->mm.page_sizes.sg || obj->mm.page_sizes.phys)) {
+		pr_err("unexpected residual page-size bits, should_swap=%s\n",
+		       yesno(should_swap));
 		err = -EINVAL;
 		goto out_put;
 	}
@@ -1567,26 +1568,39 @@ static int igt_shrink_thp(void *arg)
 	/*
 	 * Now that the pages are *unpinned* shrinking should invoke
-	 * shmem to truncate our pages.
+	 * shmem to truncate our pages, if we some available swap.
 	 */
-- 
2.26.3



More information about the Intel-gfx-trybot mailing list